#include "notifywidget.h"
#include <QEvent>
#include <KF5/KWindowSystem/KWindowEffects>
#include <QApplication>
#include <QDesktopWidget>
#include <QBitmap>
#include <QPainter>
#include <XdgIcon>
#include <QStyle>
#include <QDBusConnection>
#include <QDBusInterface>

NotifyWidget::NotifyWidget(QWidget *parent) : QWidget(parent)
{
    mSettings = new Nde::Settings("panel");

    this->setObjectName("NotifyWidget");
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::WindowStaysOnTopHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);
    KWindowEffects::enableBlurBehind(this->winId(), true);

    //icon layout
    m_labelIcon = new QLabel(this);
    m_labelIcon->setObjectName("label_icon");
    m_labelIcon->setFixedSize(NOTIFY_ICON_SIZE);
    QIcon icon = XdgIcon::fromTheme(QStringLiteral("cantor"));
    QPixmap pixmap = icon.pixmap(icon.actualSize(NOTIFY_ICON_SIZE));
    m_labelIcon->setPixmap(pixmap);

    m_layout_icon = new QVBoxLayout;
    m_layout_icon->setMargin(0);
    m_layout_icon->setContentsMargins(0,0,0,0);
    m_layout_icon->setSpacing(0);
    m_layout_icon->addWidget(m_labelIcon);
    m_layout_icon->addStretch();

    //text layout
    m_labelApplication = new QLabel(this);
    m_labelApplication->setObjectName("label_application");
    QFont f = m_labelApplication->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelApplication->setFont(f);
    m_labelApplication->setTextFormat(Qt::PlainText);
    m_labelApplication->setMinimumWidth(260);
    m_labelApplication->setText(tr("Power management"));
//    m_labelApplication->setStyleSheet("NotifyWidget QLabel#label_application{font-size:18px;color:rgb(255,255,255);}");

    m_labelSummary = new QLabel(this);
    m_labelSummary->setObjectName("label_summary");
    f = m_labelSummary->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelSummary->setFont(f);
    m_labelSummary->setTextFormat(Qt::RichText);
    m_labelSummary->setWordWrap(true);
    m_labelSummary->setMinimumWidth(260);
    m_labelSummary->setText(tr("Power low!"));
//    m_labelSummary->setStyleSheet("NotifyWidget QLabel#label_summary {font-size:14px; color:rgb(255,255,255);}");

    m_labelBody = new QLabel(this);
    m_labelBody->setObjectName("label_body");
    f = m_labelBody->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelBody->setFont(f);
    m_labelBody->setTextFormat(Qt::RichText);
    m_labelBody->setWordWrap(true);
    m_labelBody->setOpenExternalLinks(true);
    m_labelBody->setMinimumWidth(260);
//    m_labelBody->setStyleSheet("NotifyWidget QLabel#label_body {font-size:14px;color:rgb(255,255,255)}");

    m_cancel_btn = new QPushButton(this);
    m_cancel_btn->setObjectName("btn_cancel");
    m_cancel_btn->setText(tr("cancel"));
    m_cancel_btn->setMaximumWidth(150);
    m_confirm_btn = new QPushButton(this);
    m_confirm_btn->setObjectName("btn_confirm");
    m_confirm_btn->setText(tr("confirm"));
    m_confirm_btn->setMaximumWidth(80);
    connect(m_cancel_btn, &QPushButton::clicked, this, &NotifyWidget::cancelClicked);
    connect(m_confirm_btn, &QPushButton::clicked, this, &NotifyWidget::confirmClicked);

    m_contry_layout = new QHBoxLayout;
    m_contry_layout->setMargin(0);
    m_contry_layout->setSpacing(20);
    m_contry_layout->setContentsMargins(0,0,0,0);
    m_contry_layout->addWidget(m_cancel_btn);
    m_contry_layout->addWidget(m_confirm_btn);

    m_layout_text = new QVBoxLayout;
    m_layout_text->setMargin(0);
    m_layout_text->setSpacing(5);
    m_layout_text->setContentsMargins(0,0,0,0);
    m_layout_text->addWidget(m_labelApplication);
    m_layout_text->addWidget(m_labelSummary);
    m_layout_text->addWidget(m_labelBody);
    m_layout_text->addLayout(m_contry_layout);
    m_layout_text->addStretch(0);

    mBackground = new QWidget(this);
    mBackground->setObjectName("NotifyPopupBackgroundWidget");
    //main layout
    m_layout = new QHBoxLayout;
    m_layout->setContentsMargins(12,16,12,16);
    m_layout->setSpacing(10);
    m_layout->addLayout(m_layout_icon);
    m_layout->addLayout(m_layout_text);
    m_layout->addStretch(0);
    mBackground->setLayout(m_layout);

    QHBoxLayout * layout = new QHBoxLayout;
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);
    layout->addWidget(mBackground);
    this->setLayout(layout);

    m_postionMap["Bottom"] = PositionBottom;
    m_postionMap["Top"] = PositionTop;
    m_postionMap["Left"] = PositionLeft;
    m_postionMap["Right"] = PositionRight;

    QDBusConnection::sessionBus().connect(QString::fromLatin1("org.nde.session"),
                                          QString::fromLatin1("/NdeSession"),
                                          QString::fromLatin1("org.nde.session"),
                                          QString::fromLatin1("compositorOpenStateChanged"),
                                          this, SLOT(onCompositorOpenChanged(bool)));

    QDBusInterface * compositer = new QDBusInterface(QString::fromLatin1("org.nde.session"),
                                                     QString::fromLatin1("/NdeSession"),
                                                     QString::fromLatin1("org.nde.session"),
                                                     QDBusConnection::sessionBus());
    if(compositer->isValid())
    {
        QDBusMessage reply = compositer->call("getCompositorOpen");
        if(!reply.arguments().isEmpty())
        {
            mCompositorOpen = reply.arguments().first().toBool();
            mBackground->setProperty("compositor", mCompositorOpen);
        }
    }

//    QColor color = mSettings->value("background-color").value<QColor>();
//    int opacity = mSettings->value("opacity").toInt();

//    QString str;
//    str = QString("%1,%2,%3,%4")
//            .arg(color.red())
//            .arg(color.green())
//            .arg(color.blue())
//            .arg(opacity/100.0);
//    setStyleSheet(QString("NotifyWidget { background-color:rgba(" + str +");}"
//                          "NotifyWidget QPushButton {font-size:14px; color:rgb(255,255,255); background-color:rgba(" + str +");}"
//                          "NotifyWidget QPushButton#btn_cancel:hover, QPushButton#btn_confirm:hover { color:#2c9dfc;}"));

}

NotifyWidget::~NotifyWidget()
{

}

void NotifyWidget::setValues(QString nm)
{
    bool isScreensaver = false;
    if(isScreensaver) return;


    this->layout()->setEnabled(false);

    QPixmap pixmap("nde-powermanagement");
    m_labelIcon->setPixmap(pixmap);

    m_labelApplication->setText(tr("nde-powermanagement"));
    //简单描述
    m_labelSummary->setText("power low!");

    //详细描述
    QString formatted("hello");
    m_labelBody->setText(formatted.replace(QL1C('\n'), QStringLiteral("<br/>")));

    //动作
    //解析actions中的动作(actions<<acion<<desc，删除动作的描述
//    while(m_btn_actions.count())
//    {
//        QToolButton * btn = m_btn_actions.takeFirst();
//        m_layout_actions->removeWidget(btn);
//        delete  btn;
//    }
    m_frame_actions->setVisible(m_btn_actions.count());


//    this->layout()->setEnabled(true);

//    this->show();
//    this->adjustSize();
//    this->activateWindow();
}

bool NotifyWidget::event(QEvent *event)
{
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
        }
    }
    return QWidget::event(event);
}

void NotifyWidget::paintEvent(QPaintEvent * event)
{
    QRect rect = this->rect().adjusted(0,0,-1,-1);
    QBitmap bmp(rect.size());
    bmp.fill();
    QPainter painter(&bmp);
    painter.setBrush(Qt::black);
    painter.setRenderHints(QPainter::HighQualityAntialiasing );
    painter.drawRoundedRect(bmp.rect(),4,4);
    setMask(bmp);

}

void NotifyWidget::updateShow(QString doType, QString bodyText)
{
    m_labelBody->setText(bodyText);
    if(doType.isEmpty())
    {
        m_cancel_btn->hide();
        m_confirm_btn->show();
    }
    else
    {
        m_cancel_btn->setText(tr("cancel") + doType);
        m_cancel_btn->show();
        m_confirm_btn->hide();
    }

    update();

    this->show();
    this->adjustSize();
//    this->activateWindow();

    //计算位置
    QRect rect = qApp->desktop()->screenGeometry();
    int pos_x,pos_y;
    mSettings->beginGroup("panel1");
    QString position = mSettings->value("position").toString();
    int panelSize = mSettings->value("panelSize").toInt();
    mSettings->endGroup();
    switch (m_postionMap[position])
    {
    case PositionTop:
        rect.adjust(0, panelSize,0,0);
        pos_x = rect.width()-this->width();
        pos_y = panelSize + 4;
        break;
    case PositionBottom:
        rect.adjust(0,0,0,-panelSize);
        pos_x = rect.width()-this->width();
        pos_y = rect.height()-this->height()-4;
        break;

    case PositionLeft:
        rect.adjust(panelSize,0,0,0);
        pos_x = panelSize + 4;
        pos_y = rect.height()-this->height();
        break;

    case PositionRight:
        rect.adjust(0,0,-panelSize,0);
        pos_x = rect.width()-this->width()-4;
        pos_y = rect.height()-this->height();
        break;
    }
    this->move(pos_x,pos_y);
}

void NotifyWidget::cancelClicked()
{
    this->hide();
    Q_EMIT cancelAction();
}

void NotifyWidget::confirmClicked()
{
    this->hide();
}

void NotifyWidget::onCompositorOpenChanged(bool open)
{
    if(open == mCompositorOpen) return;

    mCompositorOpen = open;
    mBackground->setProperty("compositor", mCompositorOpen);
    mBackground->style()->unpolish(mBackground);
    mBackground->style()->polish(mBackground);
}
