#ifndef NOTIFYWIDGET_H
#define NOTIFYWIDGET_H

#include <QWidget>
#include <QHBoxLayout>
#include <QLabel>
#include <QToolButton>
#include <Nde/Settings>
#include <QPushButton>

#define NOTIFY_ICON_SIZE QSize(32, 32)

class NotifyWidget : public QWidget
{
    Q_OBJECT
public:
    explicit NotifyWidget(QWidget *parent = nullptr);
    ~NotifyWidget();
    /**
     * @brief Specifies the position of the panel on screen.
     */
    enum Position{
        PositionBottom, //!< The bottom side of the screen.
        PositionTop,    //!< The top side of the screen.
        PositionLeft,   //!< The left side of the screen.
        PositionRight   //!< The right side of the screen.
    };
    void setValues(QString nm);

    void updateShow(QString doType = "", QString bodyText = "");

Q_SIGNALS:
    void triggered();
    void cancelAction();

protected:
    bool event(QEvent *event) override;
    void paintEvent(QPaintEvent * event);

private Q_SLOTS:
    void cancelClicked();
    void confirmClicked();
    //特效开关变化
    void onCompositorOpenChanged(bool open);

private:
    Nde::Settings * mSettings;

    bool mCompositorOpen; //特效开关
    Position mCurPanelPos; //位置
//    int mOpacity; //背景透明度
//    QColor mBackgroundColor; //背景色

    QWidget * mBackground; //实际背景
    //界面元素
    QHBoxLayout * m_layout;
    QLabel * m_labelIcon;
    QVBoxLayout * m_layout_icon;
    QLabel * m_labelApplication;
    QLabel * m_labelSummary;
    QLabel * m_labelBody;
    QVBoxLayout * m_layout_text;
    QToolButton * m_btn_close;
    QVBoxLayout * m_layout_btn;
    QPushButton * m_cancel_btn;
    QPushButton * m_confirm_btn;
    QHBoxLayout * m_contry_layout;

    QFrame * m_frame_actions;
    QList<QToolButton*> m_btn_actions;
    QGridLayout * m_layout_actions;
    QMap<QString, Position> m_postionMap;
};

#endif // NOTIFYWIDGET_H
